@@ -1,32 +1,19 @@
-# -*- test-case-name: twisted.test.test_util -*-
-
-# Twisted, the Framework of Your Internet
-# Copyright (C) 2001 Matthew W. Lefkowitz
-#
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of version 2.1 of the GNU Lesser General Public
-# License as published by the Free Software Foundation.
-#
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-from __future__ import nested_scopes, generators
-
-__version__ = '$Revision: 1.51 $'[11:-2]
-
-import os
-import sys
-import hmac
-import errno
+# -*- test-case-name: twisted.python.test.test_util -*-
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# See LICENSE for details.
 
+import os, sys, hmac, errno, new, inspect, warnings
+try:
+    import pwd, grp
+except ImportError:
+    pwd = grp = None
+try:
+    from os import setgroups, getgroups
+except ImportError:
+    setgroups = getgroups = None
 from UserDict import UserDict
 
+
 class InsensitiveDict:
     """Dictionary, that has case-insensitive keys.
 
@@ -281,7 +268,9 @@ def _getpass(prompt):
     except EOFError:
         raise KeyboardInterrupt
 
-def getPassword(prompt = 'Password: ', confirm = 0, forceTTY = 0):
+def getPassword(prompt = 'Password: ', confirm = 0, forceTTY = 0,
+                confirmPrompt = 'Confirm password: ',
+                mismatchMessage = "Passwords don't match."):
     """Obtain a password by prompting or from stdin.
 
     If stdin is a terminal, prompt for a new password, and confirm (if
@@ -291,7 +280,7 @@ def getPassword(prompt = 'Password: ', confirm = 0, forceTTY = 0):
     If stdin is not a terminal, and C{forceTTY} is not true, read in a line
     and use it as the password, less the trailing newline, if any.  If
     C{forceTTY} is true, attempt to open a tty and prompt for the password
-    using it.  Raise a RuntimeException if this is not possible.
+    using it.  Raise a RuntimeError if this is not possible.
 
     @returns: C{str}
     """
@@ -305,7 +294,7 @@ def getPassword(prompt = 'Password: ', confirm = 0, forceTTY = 0):
                     old = sys.stdin, sys.stdout
                     sys.stdin = sys.stdout = open('/dev/tty', 'r+')
                 except:
-                    raise RuntimeException, "Cannot obtain a TTY"
+                    raise RuntimeError("Cannot obtain a TTY")
             else:
                 password = sys.stdin.readline()
                 if password[-1] == '\n':
@@ -316,11 +305,11 @@ def getPassword(prompt = 'Password: ', confirm = 0, forceTTY = 0):
             try1 = _getpass(prompt)
             if not confirm:
                 return try1
-            try2 = _getpass("Confirm: ")
+            try2 = _getpass(confirmPrompt)
             if try1 == try2:
                 return try1
             else:
-                sys.stderr.write("Passwords don't match.\n")
+                sys.stderr.write(mismatchMessage + "\n")
     finally:
         if old:
             sys.stdin.close()
@@ -328,7 +317,6 @@ def getPassword(prompt = 'Password: ', confirm = 0, forceTTY = 0):
 
 
 def dict(*a, **k):
-    import warnings
     import __builtin__
     warnings.warn('twisted.python.util.dict is deprecated.  Use __builtin__.dict instead')
     return __builtin__.dict(*a, **k)
@@ -344,8 +332,9 @@ def str_xor(s, b):
     return ''.join([chr(ord(c) ^ b) for c in s])
 
 def keyed_md5(secret, challenge):
-    """Create the keyed MD5 string for the given secret and challenge."""
-    import warnings
+    """
+    Create the keyed MD5 string for the given secret and challenge.
+    """
     warnings.warn(
         "keyed_md5() is deprecated.  Use the stdlib module hmac instead.",
         DeprecationWarning, stacklevel=2
@@ -470,13 +459,13 @@ class IntervalDifferential:
     Given a list of intervals, generate the amount of time to sleep between
     \"instants\".
 
-    For example, given 7, 11 and 13, the three (infinite) sequences
+    For example, given 7, 11 and 13, the three (infinite) sequences::
 
         7 14 21 28 35 ...
         11 22 33 44 ...
         13 26 39 52 ...
 
-    will be generated, merged, and used to produce:
+    will be generated, merged, and used to produce::
 
         (7, 0) (4, 1) (2, 2) (1, 0) (7, 0) (1, 1) (4, 2) (2, 0) (5, 1) (2, 0)
 
@@ -487,7 +476,7 @@ class IntervalDifferential:
     def __init__(self, intervals, default=60):
         """
         @type intervals: C{list} of C{int}, C{long}, or C{float} param
-        @intervals: The intervals between instants.
+        @param intervals: The intervals between instants.
 
         @type default: C{int}, C{long}, or C{float}
         @param default: The duration to generate if the intervals list
@@ -553,47 +542,135 @@ class FancyStrMixin:
         return ''.join(r)
     __repr__ = __str__
 
+
+
 class FancyEqMixin:
     compareAttributes = ()
     def __eq__(self, other):
         if not self.compareAttributes:
             return self is other
-        #XXX Maybe get rid of this, and rather use hasattr()s
-        if not isinstance(other, self.__class__):
-            return False
-        for attr in self.compareAttributes:
-            if not getattr(self, attr) == getattr(other, attr):
-                return False
-        return True
+        if isinstance(self, other.__class__):
+            return (
+                [getattr(self, name) for name in self.compareAttributes] ==
+                [getattr(other, name) for name in self.compareAttributes])
+        return NotImplemented
+
+
+    def __ne__(self, other):
+        result = self.__eq__(other)
+        if result is NotImplemented:
+            return result
+        return not result
+
+
 
 def dsu(list, key):
+    """
+    decorate-sort-undecorate (aka "Schwartzian transform")
+
+    DEPRECATED. Use the built-in C{sorted()} instead.
+    """
+    warnings.warn(("dsu is deprecated since Twisted 10.1. "
+        "Use the built-in sorted() instead."), DeprecationWarning, stacklevel=2)
     L2 = [(key(e), i, e) for (i, e) in zip(range(len(list)), list)]
     L2.sort()
     return [e[2] for e in L2]
 
+
 try:
-    import pwd, grp
-    from os import setgroups, getgroups
+    from twisted.python._initgroups import initgroups as _c_initgroups
+except ImportError:
+    _c_initgroups = None
+
+
+
+if pwd is None or grp is None or setgroups is None or getgroups is None:
     def initgroups(uid, primaryGid):
+        """
+        Do nothing.
+
+        Underlying platform support require to manipulate groups is missing.
+        """
+else:
+    # Fallback to the inefficient Python version
+    def _setgroups_until_success(l):
+        while(1):
+            # NASTY NASTY HACK (but glibc does it so it must be okay):
+            # In case sysconfig didn't give the right answer, find the limit
+            # on max groups by just looping, trying to set fewer and fewer
+            # groups each time until it succeeds.
+            try:
+                setgroups(l)
+            except ValueError:
+                # This exception comes from python itself restricting
+                # number of groups allowed.
+                if len(l) > 1:
+                    del l[-1]
+                else:
+                    raise
+            except OSError, e:
+                if e.errno == errno.EINVAL and len(l) > 1:
+                    # This comes from the OS saying too many groups
+                    del l[-1]
+                else:
+                    raise
+            else:
+                # Success, yay!
+                return
+
+    def initgroups(uid, primaryGid):
+        """
+        Initializes the group access list.
+
+        If the C extension is present, we're calling it, which in turn calls
+        initgroups(3).
+        
+        If not, this is done by reading the group database /etc/group and using
+        all groups of which C{uid} is a member.  The additional group
+        C{primaryGid} is also added to the list.
+
+        If the given user is a member of more than C{NGROUPS}, arbitrary
+        groups will be silently discarded to bring the number below that
+        limit.
+
+        @type uid: C{int}
+        @param uid: The UID for which to look up group information.
+
+        @type primaryGid: C{int} or C{NoneType}
+        @param primaryGid: If provided, an additional GID to include when
+            setting the groups.
+        """
+        if _c_initgroups is not None:
+            return _c_initgroups(pwd.getpwuid(uid)[0], primaryGid)
+        try:
+            # Try to get the maximum number of groups
+            max_groups = os.sysconf("SC_NGROUPS_MAX")
+        except:
+            # No predefined limit
+            max_groups = 0
+
         username = pwd.getpwuid(uid)[0]
-        l=[primaryGid]
+        l = []
+        if primaryGid is not None:
+            l.append(primaryGid)
         for groupname, password, gid, userlist in grp.getgrall():
             if username in userlist:
                 l.append(gid)
+                if len(l) == max_groups:
+                    break # No more groups, ignore any more
         try:
-            setgroups(l)
+            _setgroups_until_success(l)
         except OSError, e:
+            # We might be able to remove this code now that we
+            # don't try to setgid/setuid even when not asked to.
             if e.errno == errno.EPERM:
-                groups = getgroups()
                 for g in getgroups():
                     if g not in l:
                         raise
             else:
                 raise
 
-except:
-    def initgroups(uid, primaryGid):
-        pass
+
 
 def switchUID(uid, gid, euid=False):
     if euid:
@@ -602,14 +679,298 @@ def switchUID(uid, gid, euid=False):
     else:
         setuid = os.setuid
         setgid = os.setgid
-    setgid(gid)
-    initgroups(uid, gid)
-    setuid(uid)
+    if gid is not None:
+        setgid(gid)
+    if uid is not None:
+        initgroups(uid, gid)
+        setuid(uid)
+
+
+class SubclassableCStringIO(object):
+    """A wrapper around cStringIO to allow for subclassing"""
+    __csio = None
+
+    def __init__(self, *a, **kw):
+        from cStringIO import StringIO
+        self.__csio = StringIO(*a, **kw)
+
+    def __iter__(self):
+        return self.__csio.__iter__()
+
+    def next(self):
+        return self.__csio.next()
+
+    def close(self):
+        return self.__csio.close()
+
+    def isatty(self):
+        return self.__csio.isatty()
+
+    def seek(self, pos, mode=0):
+        return self.__csio.seek(pos, mode)
+
+    def tell(self):
+        return self.__csio.tell()
+
+    def read(self, n=-1):
+        return self.__csio.read(n)
+
+    def readline(self, length=None):
+        return self.__csio.readline(length)
+
+    def readlines(self, sizehint=0):
+        return self.__csio.readlines(sizehint)
+
+    def truncate(self, size=None):
+        return self.__csio.truncate(size)
+
+    def write(self, s):
+        return self.__csio.write(s)
+
+    def writelines(self, list):
+        return self.__csio.writelines(list)
+
+    def flush(self):
+        return self.__csio.flush()
+
+    def getvalue(self):
+        return self.__csio.getvalue()
+
+def moduleMovedForSplit(origModuleName, newModuleName, moduleDesc,
+                        projectName, projectURL, globDict):
+    """
+    No-op function; only present for backwards compatibility.  There is no
+    reason to call this function.
+    """
+    warnings.warn(
+        "moduleMovedForSplit is deprecated since Twisted 9.0.",
+        DeprecationWarning, stacklevel=2)
+
+
+def untilConcludes(f, *a, **kw):
+    while True:
+        try:
+            return f(*a, **kw)
+        except (IOError, OSError), e:
+            if e.args[0] == errno.EINTR:
+                continue
+            raise
+
+_idFunction = id
+
+def setIDFunction(idFunction):
+    """
+    Change the function used by L{unsignedID} to determine the integer id value
+    of an object.  This is largely useful for testing to give L{unsignedID}
+    deterministic, easily-controlled behavior.
+
+    @param idFunction: A function with the signature of L{id}.
+    @return: The previous function being used by L{unsignedID}.
+    """
+    global _idFunction
+    oldIDFunction = _idFunction
+    _idFunction = idFunction
+    return oldIDFunction
+
+
+# A value about twice as large as any Python int, to which negative values
+# from id() will be added, moving them into a range which should begin just
+# above where positive values from id() leave off.
+_HUGEINT = (sys.maxint + 1L) * 2L
+def unsignedID(obj):
+    """
+    Return the id of an object as an unsigned number so that its hex
+    representation makes sense.
+
+    This is mostly necessary in Python 2.4 which implements L{id} to sometimes
+    return a negative value.  Python 2.3 shares this behavior, but also
+    implements hex and the %x format specifier to represent negative values as
+    though they were positive ones, obscuring the behavior of L{id}.  Python
+    2.5's implementation of L{id} always returns positive values.
+    """
+    rval = _idFunction(obj)
+    if rval < 0:
+        rval += _HUGEINT
+    return rval
+
+
+def mergeFunctionMetadata(f, g):
+    """
+    Overwrite C{g}'s name and docstring with values from C{f}.  Update
+    C{g}'s instance dictionary with C{f}'s.
+
+    To use this function safely you must use the return value. In Python 2.3,
+    L{mergeFunctionMetadata} will create a new function. In later versions of
+    Python, C{g} will be mutated and returned.
+
+    @return: A function that has C{g}'s behavior and metadata merged from
+        C{f}.
+    """
+    try:
+        g.__name__ = f.__name__
+    except TypeError:
+        try:
+            merged = new.function(
+                g.func_code, g.func_globals,
+                f.__name__, inspect.getargspec(g)[-1],
+                g.func_closure)
+        except TypeError:
+            pass
+    else:
+        merged = g
+    try:
+        merged.__doc__ = f.__doc__
+    except (TypeError, AttributeError):
+        pass
+    try:
+        merged.__dict__.update(g.__dict__)
+        merged.__dict__.update(f.__dict__)
+    except (TypeError, AttributeError):
+        pass
+    merged.__module__ = f.__module__
+    return merged
+
+
+def nameToLabel(mname):
+    """
+    Convert a string like a variable name into a slightly more human-friendly
+    string with spaces and capitalized letters.
+
+    @type mname: C{str}
+    @param mname: The name to convert to a label.  This must be a string
+    which could be used as a Python identifier.  Strings which do not take
+    this form will result in unpredictable behavior.
+
+    @rtype: C{str}
+    """
+    labelList = []
+    word = ''
+    lastWasUpper = False
+    for letter in mname:
+        if letter.isupper() == lastWasUpper:
+            # Continuing a word.
+            word += letter
+        else:
+            # breaking a word OR beginning a word
+            if lastWasUpper:
+                # could be either
+                if len(word) == 1:
+                    # keep going
+                    word += letter
+                else:
+                    # acronym
+                    # we're processing the lowercase letter after the acronym-then-capital
+                    lastWord = word[:-1]
+                    firstLetter = word[-1]
+                    labelList.append(lastWord)
+                    word = firstLetter + letter
+            else:
+                # definitely breaking: lower to upper
+                labelList.append(word)
+                word = letter
+        lastWasUpper = letter.isupper()
+    if labelList:
+        labelList[0] = labelList[0].capitalize()
+    else:
+        return mname.capitalize()
+    labelList.append(word)
+    return ' '.join(labelList)
+
+
+
+def uidFromString(uidString):
+    """
+    Convert a user identifier, as a string, into an integer UID.
+
+    @type uid: C{str}
+    @param uid: A string giving the base-ten representation of a UID or the
+        name of a user which can be converted to a UID via L{pwd.getpwnam}.
+
+    @rtype: C{int}
+    @return: The integer UID corresponding to the given string.
+
+    @raise ValueError: If the user name is supplied and L{pwd} is not
+        available.
+    """
+    try:
+        return int(uidString)
+    except ValueError:
+        if pwd is None:
+            raise
+        return pwd.getpwnam(uidString)[2]
+
+
+
+def gidFromString(gidString):
+    """
+    Convert a group identifier, as a string, into an integer GID.
+
+    @type uid: C{str}
+    @param uid: A string giving the base-ten representation of a GID or the
+        name of a group which can be converted to a GID via L{grp.getgrnam}.
+
+    @rtype: C{int}
+    @return: The integer GID corresponding to the given string.
+
+    @raise ValueError: If the group name is supplied and L{grp} is not
+        available.
+    """
+    try:
+        return int(gidString)
+    except ValueError:
+        if grp is None:
+            raise
+        return grp.getgrnam(gidString)[2]
+
+
+
+def runAsEffectiveUser(euid, egid, function, *args, **kwargs):
+    """
+    Run the given function wrapped with seteuid/setegid calls.
+
+    This will try to minimize the number of seteuid/setegid calls, comparing
+    current and wanted permissions
+
+    @param euid: effective UID used to call the function.
+    @type euid: C{int}
+
+    @type egid: effective GID used to call the function.
+    @param egid: C{int}
+
+    @param function: the function run with the specific permission.
+    @type function: any callable
+
+    @param *args: arguments passed to C{function}
+    @param **kwargs: keyword arguments passed to C{function}
+    """
+    uid, gid = os.geteuid(), os.getegid()
+    if uid == euid and gid == egid:
+        return function(*args, **kwargs)
+    else:
+        if uid != 0 and (uid != euid or gid != egid):
+            os.seteuid(0)
+        if gid != egid:
+            os.setegid(egid)
+        if euid != 0 and (euid != uid or gid != egid):
+            os.seteuid(euid)
+        try:
+            return function(*args, **kwargs)
+        finally:
+            if euid != 0 and (uid != euid or gid != egid):
+                os.seteuid(0)
+            if gid != egid:
+                os.setegid(gid)
+            if uid != 0 and (uid != euid or gid != egid):
+                os.seteuid(uid)
+
+
 
 __all__ = [
     "uniquify", "padTo", "getPluginDirs", "addPluginDir", "sibpath",
     "getPassword", "dict", "println", "keyed_md5", "makeStatBar",
-    "OrderedDict", "spewer", "searchupwards", "LineLog", "raises",
-    "IntervalDifferential", "FancyStrMixin", "FancyEqMixin", "dsu",
-    "switchUID"
+    "OrderedDict", "InsensitiveDict", "spewer", "searchupwards", "LineLog",
+    "raises", "IntervalDifferential", "FancyStrMixin", "FancyEqMixin",
+    "dsu", "switchUID", "SubclassableCStringIO", "moduleMovedForSplit",
+    "unsignedID", "mergeFunctionMetadata", "nameToLabel", "uidFromString",
+    "gidFromString", "runAsEffectiveUser", "moduleMovedForSplit",
 ]
