@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -6,34 +6,20 @@ Tests for L{twisted.application} and its interaction with
 L{twisted.persisted.sob}.
 """
 
-import sys, copy, os, pickle
+import copy, os, pickle
 from StringIO import StringIO
 
 from twisted.trial import unittest, util
 from twisted.application import service, internet, app
 from twisted.persisted import sob
 from twisted.python import usage
-from twisted.python.util import sibpath
-from twisted.python.runtime import platform
 from twisted.internet import interfaces, defer
 from twisted.protocols import wire, basic
 from twisted.internet import protocol, reactor
-from twisted.internet.utils import getProcessOutputAndValue
 from twisted.application import reactors
 from twisted.test.proto_helpers import MemoryReactor
 
 
-oldAppSuppressions = [util.suppress(message='twisted.internet.app is deprecated',
-                                    category=DeprecationWarning)]
-
-skipWindowsNopywin32 = None
-if platform.isWindows():
-    try:
-        import win32process
-    except ImportError:
-        skipWindowsNopywin32 = ("On windows, spawnProcess is not available "
-                                "in the absence of win32process.")
-
 class Dummy:
     processName=None
 
@@ -294,21 +280,6 @@ class TestAppSupport(unittest.TestCase):
                 appl2 = service.loadApplication("converttest.out", outstyle)
                 self.assertEqual(service.IService(appl2).name, "lala")
 
-    def test_getLogFile(self):
-        """
-        Test L{app.getLogFile}, veryfying the LogFile instance it returns.
-        """
-        os.mkdir("logfiledir")
-        l = app.getLogFile(os.path.join("logfiledir", "lala"))
-        self.assertEqual(l.path,
-                         os.path.abspath(os.path.join("logfiledir", "lala")))
-        self.assertEqual(l.name, "lala")
-        self.assertEqual(l.directory, os.path.abspath("logfiledir"))
-
-    test_getLogFile.suppress = [
-        util.suppress(message="app.getLogFile is deprecated. Use "
-                      "twisted.python.logfile.LogFile.fromFullPath instead",
-                      category=DeprecationWarning)]
 
     def test_startApplication(self):
         appl = service.Application("lala")
@@ -518,10 +489,44 @@ class TestInternet2(unittest.TestCase):
             [o.value.__class__ for o in self.flushLoggedErrors(ZeroDivisionError)]))
         return d
 
-    def testEverythingThere(self):
+
+    def test_genericServerDeprecated(self):
+        """
+        Instantiating L{GenericServer} emits a deprecation warning.
+        """
+        internet.GenericServer()
+        warnings = self.flushWarnings(
+            offendingFunctions=[self.test_genericServerDeprecated])
+        self.assertEquals(
+            warnings[0]['message'],
+            'GenericServer was deprecated in Twisted 10.1.')
+        self.assertEquals(warnings[0]['category'], DeprecationWarning)
+        self.assertEquals(len(warnings), 1)
+
+
+    def test_genericClientDeprecated(self):
+        """
+        Instantiating L{GenericClient} emits a deprecation warning.
+        """
+        internet.GenericClient()
+        warnings = self.flushWarnings(
+            offendingFunctions=[self.test_genericClientDeprecated])
+        self.assertEquals(
+            warnings[0]['message'],
+            'GenericClient was deprecated in Twisted 10.1.')
+        self.assertEquals(warnings[0]['category'], DeprecationWarning)
+        self.assertEquals(len(warnings), 1)
+
+
+    def test_everythingThere(self):
+        """
+        L{twisted.application.internet} dynamically defines a set of
+        L{service.Service} subclasses that in general have corresponding
+        reactor.listenXXX or reactor.connectXXX calls.
+        """
         trans = 'TCP UNIX SSL UDP UNIXDatagram Multicast'.split()
         for tran in trans[:]:
-            if not getattr(interfaces, "IReactor"+tran)(reactor, None):
+            if not getattr(interfaces, "IReactor" + tran)(reactor, None):
                 trans.remove(tran)
         if interfaces.IReactorArbitrary(reactor, None) is not None:
             trans.insert(0, "Generic")
@@ -529,13 +534,32 @@ class TestInternet2(unittest.TestCase):
             for side in 'Server Client'.split():
                 if tran == "Multicast" and side == "Client":
                     continue
-                self.assert_(hasattr(internet, tran+side))
-                method = getattr(internet, tran+side).method
+                self.assertTrue(hasattr(internet, tran + side))
+                method = getattr(internet, tran + side).method
                 prefix = {'Server': 'listen', 'Client': 'connect'}[side]
-                self.assert_(hasattr(reactor, prefix+method) or
+                self.assertTrue(hasattr(reactor, prefix + method) or
                         (prefix == "connect" and method == "UDP"))
-                o = getattr(internet, tran+side)()
-                self.assertEqual(service.IService(o), o)
+                o = getattr(internet, tran + side)()
+                self.assertEquals(service.IService(o), o)
+    test_everythingThere.suppress = [
+        util.suppress(message='GenericServer was deprecated in Twisted 10.1.',
+                      category=DeprecationWarning),
+        util.suppress(message='GenericClient was deprecated in Twisted 10.1.',
+                      category=DeprecationWarning),
+        util.suppress(message='twisted.internet.interfaces.IReactorArbitrary was '
+                      'deprecated in Twisted 10.1.0: See IReactorFDSet.')]
+
+
+    def test_importAll(self):
+        """
+        L{twisted.application.internet} dynamically defines L{service.Service}
+        subclasses. This test ensures that the subclasses exposed by C{__all__}
+        are valid attributes of the module.
+        """
+        for cls in internet.__all__:
+            self.assertTrue(
+                hasattr(internet, cls),
+                '%s not importable from twisted.application.internet' % (cls,))
 
 
     def test_reactorParametrizationInServer(self):
@@ -851,34 +875,3 @@ class PluggableReactorTestCase(unittest.TestCase):
                                ['--reactor', 'fakereactortest', 'subcommand'])
         self.assertIn(message, e.args[0])
         self.assertIn("help-reactors", e.args[0])
-
-
-    def test_qtStub(self):
-        """
-        Test that installing qtreactor when it's absent fails properly.
-        """
-        scriptPath = sibpath(__file__, "app_qtstub.py")
-        def _checkOutput((output, err, code)):
-            self.failIf(output, output)
-        result = getProcessOutputAndValue(
-            sys.executable,
-            args=(sys.executable, scriptPath),
-            env=None)
-        result.addCallback(_checkOutput)
-        return result
-    test_qtStub.skip = skipWindowsNopywin32
-
-
-
-class ReportProfileTestCase(unittest.TestCase):
-    """
-    Tests for L{app.reportProfile}.
-    """
-
-    def test_deprecation(self):
-        """
-        Check that L{app.reportProfile} prints a warning and does nothing else.
-        """
-        self.assertWarns(DeprecationWarning,
-            "reportProfile is deprecated and a no-op since Twisted 8.0.",
-            app.__file__, app.reportProfile, None, None)
